--[[

	(C) Copyright 2010 Dustin Atwood aka "Ryknzu"(Ryknzu@gmail.com)
	
	See License.htm for license terms.
]]
--[[

Chitchat

Chitchat is an addon that works as a in-game answering machine. You can 
choose when Chitchat will begin recording incoming whispers. While recording 
choose how Chitchat will notify you of a whisper. All incoming whispers are 
saved locally and can be viewed at anytime through an easy to use interface. 
Have a fully customized experience with many easy to understand options.

I appreciate any feedback, please email me at ryknzu@gmail.com with your questions, concerns or suggestions.

Coming soon:

- Blacklist Settings { Required level to whisper, Auto-reply message }
- Autoreply key-words 
- Submode forward Whispers

- Modular Design
- Ability to toggle multiple modes, 'AFK/DND' and 'In Town'
]]

--------------------
-- Register with Ace
Chitchat = LibStub("AceAddon-3.0"):NewAddon("Chitchat", 
	"AceConsole-3.0", 
	"AceEvent-3.0",  
	"AceHook-3.0",
	"AceTimer-3.0")
	
-- Load Localization
local L = LibStub("AceLocale-3.0"):GetLocale("Chitchat")
-- Initialize minimap icon
local icon = LibStub("LibDBIcon-1.0")

-----------------
--| Constants |--
-----------------
CHITCHAT_VERSION = GetAddOnMetadata("Chitchat", "Version")
BINDING_HEADER_CHITCHAT = L["CHITCHAT"]
BINDING_NAME_CHITCHAT = L["TOGGLE"]
MODE_OFF = 1
MODE_ON = 2
MODE_AFK = 3
MODE_DND = 4
MODE_AFKDND = 5
MODE_TOWN = 6
MODE_COMBAT = 7
MODE_COMBAT_ALL = 1
MODE_COMBAT_ELITE = 2
MODE_COMBAT_BOSS = 3
MODE_COMBAT_BOTH = 4
SUBMODE_OFF = 1
SUBMODE_HIDE = 2
SUBMODE_NOTIFY = 3
SUBMODE_FORWARD = 4
HYPERLINK_REF = format("%s:%s", L["CHITCHAT"], L["VIEW"])
HYPERLINK = format("|c00FFFF00|H%s|h%s|h|r",HYPERLINK_REF, L["VIEW_MSG"])
HYPERLINK_CHITCHAT = format("|c00FFFF00|H%s|h%s: |h|r",HYPERLINK_REF, L["CHITCHAT"])

---------------
-- Option Table
local Options = 
{
    name = L["CHITCHAT"],
    handler = Chitchat,
    type  = "group",
    args = 
	{
		addon_desc = 
        { -- Sets the Answering Machine mode description
			name = L["OPT_ADDON_DESC"],
			order = 1,
            type = "description",
			width = "full",
        },
		header_general = 
		{ -- General Options Header
			name = L["OPT_HEADER_GENERAL"],
			order = 2,
			type = "header",
		},
		minimap = 
		{ -- Toggles the Answering Machine minimap icon
			name = L["OPT_MINIMAP"],
			desc = L["OPT_MINIMAP_DESC"],
			order = 3,
			type = "toggle",
			width = "normal",
			get = "GetMinimap",
			set = "SetMinimap"
		},
		toggle = 
		{ -- Toggles the Answering Machine UI
			name = L["OPT_TOGGLE"],
			desc = L["OPT_TOGGLE_DESC"],
			order = 4,
			type = "execute",
			func = "ToggleChitchatFrame"
		},
		afk = 
        { -- Sets the Answering Machine AFK MESSAGE
			name = L["OPT_AFK"],
			order = 5,
            type = "toggle",
			width = "normal",
			get = "IsAFKEnabled",
			set = "ToggleAFKMessage",
        },
        afk_desc = 
        { -- Sets the Answering Machine AFK description
			name = "",
			desc = L["OPT_AFK_DESC"],
			order = 6,
            type = "input",
			disabled = function(info) return not Chitchat.db.profile.afk.enable end,
			width = "normal",
            get = "GetAFK",
            set = "SetAFK",
        },
		dnd_desc = 
        { -- Sets the Answering Machine DND description
            name = L["OPT_DND"],
			order = 7,
            type = "toggle",
			width = "normal",
			get = "IsDNDEnabled",
			set = "ToggleDNDMessage",
        },
		dnd = 
        { -- Sets the Answering Machine DND message
            name = "",
			desc = L["OPT_DND_DESC"],
			order = 8,
            type = "input",
			disabled = function(info) return not Chitchat.db.profile.dnd.enable end,
			width = "normal",
            get = "GetDND",
            set = "SetDND",
        },
		realID_desc = 
        { -- Sets the Answering Machine AFK description
			name = L["OPT_REALID_DESC"],
			order = 9,
            type = "description",
			width = "normal",
        },
        realID = 
        { -- Sets the Answering Machine AFK message
			name = L["OPT_REALID"],
			desc = L["OPT_REALID"],
			order = 10,
            type = "toggle",
			width = "normal",
			get = "IsRIDEnabled",
			set = "ToggleRIDMessage",
        },
		color_desc = 
		{ -- Toggles the Answering Machine recording
			name = L["OPT_COLOR_DESC"],
			order = 11,
            type = "description",
			width = "normal",
		},
		color = 
		{ -- Toggles the Answering Machine recording
			name = L["OPT_COLOR"],
			desc = L["OPT_COLOR_DESC"],
			order = 12,
            type = "color",
			width = "normal",
            get = "GetUnreadColor",
            set = "SetUnreadColor",
		},
		record_name_desc = 
        { -- Sets the Answering Machine Recordlist add description
			name = L["OPT_RECORDLIST"],
			order = 13,
            type = "description",
			width = "normal",
        },
        record_name_add = 
        { -- Sets the Answering Machine Recordlist message
			name = " ",
			desc = L["OPT_RECORDLIST"],
			order = 14,
            type = "input",
			width = "normal",
            get = function(info) return "" end,
            set = "AddRecordlist",
        },
		recordlist_desc = 
        { -- Sets the Answering Machine Recordlist list/remove description
			name = L["OPT_RECORDLIST_DESC"],
			order =15,
            type = "description",
			width = "normal",
        },
		recordlist = 
        { -- Shows the Answering Machine Recordlist names
			name = "Always record following names",
			desc = L["OPT_RECORDLIST_DESC"],
			order = 16,
            type = "select",
			width = "normal",
			values = "GetRecordlist",
            get = "GetRecordlist",
            set = "SetRecordlist",
			style = "dropdown",
        },
		clear_desc = {
			name = L["OPT_CLEAR_DESC"],
			order = 17,
			type = "description",
			width = "normal",
		},
		clear = {
			name = L["OPT_CLEAR"],
			name = L["OPT_CLEAR"],
			order = 18,
			type = "execute",
			func = "ClearLogs",
			confirm = true,
		},
		header_mode = 
		{
			name = L["OPT_HEADER_MODES"],
			order = 19,
			type = "header",
		},
		mode_desc = 
        { -- Sets the Answering Machine mode description
			name = L["OPT_MODE_DESC"],
			order = 20,
            type = "description",
			width = "normal",
        },
		mode = 
		{ -- Toggles the Answering Machine recording
			name = L["OPT_MODE"],
			desc = L["OPT_MODE_DESC"],
			order = 21,
            type = "select",
			width = "normal",
			values = {L["OPT_MODE_OFF"], L["OPT_MODE_ON"], L["OPT_MODE_AFK"], L["OPT_MODE_DND"], L["OPT_MODE_AFKDND"], L["OPT_MODE_TOWN"], L["OPT_MODE_COMBAT"]},
            get = "GetMode",
            set = "SetMode",
			style = "dropdown",
		},
		mode_combat_desc = 
		{
			name = L["OPT_MODE_COMBAT_MESSAGE"],
			desc = L["OPT_MODE_COMBAT_MESSAGE_DESC"],
			order = 22,
            type = "toggle",
			hidden = "VisibleModeCombat",
			width = "normal",
			get = "IsCombatAutoreply",
			set = "ToggleCombatAutoreply",
		},
		mode_combat_message = 
		{
			name = L["OPT_MODE_COMBAT"],
			desc = L["OPT_MODE_COMBAT_DESC"],
			order = 23,
            type = "input",
			hidden = "VisibleModeCombat",
			width = "normal",
			disabled = function(info) return not Chitchat.db.profile.combatAutoReply end,
			width = "normal",
            get = "GetCombatReplyMessage",
            set = "SetCombatReplyMessage",
		},
		mode_combat_message_desc = 
		{
			name = L["OPT_MODE_COMBAT_MESSAGE_DESC"],
			order = 24,
            type = "description",
			hidden = "VisibleModeCombat",
			width = "normal",
		},
		mode_combat_toggle = 
		{
			name = L["OPT_MODE_COMBAT"],
			desc = L["OPT_MODE_COMBAT_DESC"],
			order = 25,
            type = "select",
			hidden = "VisibleModeCombat",
			width = "normal",
			values = {L["OPT_MODE_COMBAT_ALL"], L["OPT_MODE_COMBAT_ELITE"], L["OPT_MODE_COMBAT_BOSS"], L["OPT_MODE_COMBAT_BOTH"]},
            get = "GetModeCombat",
            set = "SetModeCombat",
			style = "dropdown",
		},
		submode_desc = 
        { -- Sets the Answering Machine mode description
			name = L["OPT_SUBMODE_DESC"],
			order = 26,
            type = "description",
			width = "normal",
        },
		submode = 
		{ -- Toggles the Answering Machine recording
			name = L["OPT_SUBMODE"],
			desc = L["OPT_SUBMODE_DESC"],
			order = 27,
            type = "select",
			width = "normal",
			values = {L["OPT_SUBMODE_OFF"], L["OPT_SUBMODE_HIDE"], L["OPT_SUBMODE_NOTIFY"]},
            get = "GetSubmode",
            set = "SetSubmode",
			style = "dropdown",
		},
		header_blacklist = 
		{
			name = L["OPT_HEADER_BLACKLIST"],
			order = 28,
			type = "header",
			hidden = true,
		},
		blacklist_level = 
		{
			name = L["OPT_BLACKLIST_LEVEL"],
			order = 29,
			type = "range",
			hidden = true,
			width = "normal",
			min = 1,
			max = 80,
			get = "GetBlacklistLevel",
			set = "SetBlacklistLevel",
		},
		blacklist_autoreply_desc = 
        { -- Sets the Answering Machine DND description
            name = L["OPT_BLACKLIST_AUTOREPLY"],
			order = 30,
            type = "toggle",
			hidden = true,
			width = "normal",
			get = "IsBlackReplyEnabled",
			set = "ToggleBlackReplyMessage",
        },
		blacklist_autoreply = 
        { -- Sets the Answering Machine DND message
            name = "",
			desc = L["OPT_BLACKLIST_AUTOREPLY"],
			order = 32,
            type = "input",
			hidden = true,
			disabled = function(info) return not Chitchat.db.profile.blacklist.enable end,
			width = "normal",
            get = "GetBlacklistReply",
            set = "SetBlacklistReply",
        },
		options = 
		{ -- Toggles the Answering Machine option panel
			name = L["OPT_OPTIONS"],
			desc = L["OPT_OPTIONS_DESC"],
			order = -1,
			guiHidden = true,
			type = "execute",
			func = "ShowOptions",
		},
    },
}

-------------------
-- Profile Defaults
local Defaults = 
{
    profile = 
    {
		names = {},
		minimap = 
		{
			hide = false
		},
        afk = 
		{
			enable = false,
			message = L["AFK_MESSAGE_DEFAULT"]
		},
		dnd = 
		{
			enable = false,
			message = L["DND_MESSAGE_DEFAULT"]
		},
		rid = true,
		mode = MODE_AFKDND,
		submode = SUBMODE_OFF,
		combatmode = 1,
		combatAutoReply = true,
		combatReplyMessage = L["MODE_COMBAT_AUTOREPLY"],
		unread = 0,
		unreadColor = {r = 1, b = 0.5, g = 1},
		recordlist = {},
		blacklist = 
		{
			enable = false,
			min = 1,
			autoreply = true,
			message = L["BLACKLIST_REPLY"]
		},
    },
}

-----------------------------
--| INIT, ENABLE, DISABLE |--
-----------------------------
---------------
-- OnInitialize
-- Called when the Addon is Loaded
function Chitchat:OnInitialize()
	-- Loads the database
    self.db = LibStub("AceDB-3.0"):New("ChitchatDB",Defaults, "Default", "factionrealm")
	
	self:InitializeChitchatFrame()
	
    -- Registers the Option Table
    LibStub("AceConfig-3.0"):RegisterOptionsTable("Chitchat", Options)
    -- Adds options to 'Addons' Tab
    self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Chitchat",L["OPT_ADDON_TITLE"])
	
    -- Slash Commands
    self:RegisterChatCommand("chitchat", "ChatCommand")
	
	-- Setup minimap icon
	self.dataObject = LibStub("LibDataBroker-1.1"):NewDataObject("Chitchat", { 
		type = "data source",
		text = L["CHITCHAT"],
		icon = "Interface\\Icons\\INV_Misc_EngGizmos_13.blp",
		OnClick = function (frame, button)
			if button == "RightButton" then
				-- Show Options Window
				self:ShowOptions()
			elseif button == "MiddleButton" then
				-- Force Start/Stop Recording
				if(isRecording == true) then
					StopRecording()
				else
					StartRecording()
				end
			else
				-- Show Interface Window
				self:ToggleChitchatFrame()
			end
		end,
		OnTooltipShow = function (tooltip)
			self:OnTooltipShow(tooltip)
		end
	})
	icon:Register("Chitchat", self.dataObject, self.db.profile.minimap)
	
	-- Spam Control
	lastWhisper = ""
	-- is currently recording
	isRecording = false
	hyperNameIndex = -1
	hyperDateIndex = -1
	isViewingOptions = false
	combatReply = false
end

-----------
-- OnEnable
-- Called when the addon is Enabled
function Chitchat:OnEnable()	
    -- Register events
	Chitchat:RegisterEvent("PLAYER_ENTERING_WORLD")
	Chitchat:RegisterEvent("PLAYER_FLAGS_CHANGED")
	Chitchat:RegisterEvent("PLAYER_UPDATE_RESTING")
	Chitchat:RegisterEvent("PLAYER_REGEN_DISABLED")
	Chitchat:RegisterEvent("PLAYER_REGEN_ENABLED")
	ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", ChatSystemMessage)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", ChatWhisperMessage)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_WHISPER", ChatBNWhisperMessage)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_AFK", ChatStatusMessage)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_DND", ChatStatusMessage)
	self:RawHook(nil, "SetItemRef", true)
end

------------
-- OnDisable
-- Called when the addon is Disabled
function Chitchat:OnDisable()
	StopRecording()
	-- Unregister events
	Chitchat:UnregisterEvent("PLAYER_ENTERING_WORLD")
	Chitchat:UnregisterEvent("PLAYER_FLAGS_CHANGED")
	Chitchat:UnregisterEvent("PLAYER_UPDATE_RESTING")
	Chitchat:UnregisterEvent("PLAYER_REGEN_DISABLED")
	Chitchat:UnregisterEvent("PLAYER_REGEN_ENABLED")
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", ChatSystemMessage)
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER", ChatWhisperMessage)
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_AFK", ChatStatusMessage)
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_DND", ChatStatusMessage)
end


--------------------------
-----| EVENT HANDLERS |---
--------------------------

-----------------------
-- PLAYER_ENTERING_WORLD
-- Called whenever the player sees a loading screen
function Chitchat:PLAYER_ENTERING_WORLD (message, unit)
	-- Check to see if it will always be on
	if(Chitchat.db.profile.mode == MODE_ON) then
		StartRecording()
	end
	-- Checks player AFK status upon Enabling
	if(UnitIsAFK("player")) then
		if(Chitchat.db.profile.mode == MODE_AFK or Chitchat.db.profile.mode == MODE_AFKDND) then
			StartRecording()
		end
	end
	-- Checks player DND status upon Enabling
	if(UnitIsDND("player")) then
		if(Chitchat.db.profile.mode == MODE_DND or Chitchat.db.profile.mode == MODE_AFKDND) then
			StartRecording()
		end
	end
	if (IsResting()) then
		if(Chitchat.db.profile.mode == MODE_TOWN) then
			StartRecording()
		end
	end
	MissedWhisper()
end

-------------
-- SetItemRef
-- Sets the item l
function Chitchat:SetItemRef(link, text, button)
	if (link == HYPERLINK_REF) then 
		if(button == "LeftButton") then
			--Chitchat:SendMessage("CHITCHAT_MSG_HYPERLINK_OPEN", hyperNameIndex, hyperDateIndex)
			Chitchat:ToggleChitchatFrame()
		elseif(button == "RightButton") then
			InterfaceOptionsFrame_OpenToCategory(Chitchat.optionsFrame)
		end
		return nil
	end
	return self.hooks.SetItemRef(link, text, button)
end

----------------
-- OnTooltipShow
-- Tooltip when over minimap icon
function Chitchat:OnTooltipShow (tooltip)
	tooltip:AddLine(L["CHITCHAT"])
	if(isRecording) then
		tooltip:AddLine(L["RECORDING_ON"])
	else
		tooltip:AddLine(L["RECORDING_OFF"])
	end
	if (Chitchat.db.profile.unread == 1) then
		tooltip:AddLine(L["MISSED_MSG_FIRST"] .. Chitchat_UnreadColorHex() ..  Chitchat.db.profile.unread .. "|r" .. L["MISSED_MSG_LAST_SINGULAR"])
	else
		tooltip:AddLine(L["MISSED_MSG_FIRST"] .. Chitchat_UnreadColorHex() .. Chitchat.db.profile.unread .. "|r" .. L["MISSED_MSG_LAST_PLURAL"])
	end
	tooltip:AddLine(L["INSTRUCT_LEFTCLICK"])
	tooltip:AddLine(L["INSTRUCT_MIDCLICK"])
	tooltip:AddLine(L["INSTRUCT_RIGHTCLICK"])
end

-----------------------
-- PLAYER_FLAGS_CHANGED
-- Sets the afk or dnd status messages
function Chitchat:PLAYER_FLAGS_CHANGED (message, unit)
	if (unit == "player") then
		if(UnitIsAFK("player") and self.db.profile.afk.enable == true) then
			SendChatMessage(Chitchat.db.profile.afk.message , "AFK");
		elseif(UnitIsDND("player") and self.db.profile.dnd.enable == true) then
			SendChatMessage(Chitchat.db.profile.dnd.message , "DND");
		end
	end
end

-----------------------
-- PLAYER_UPDATE_RESTING
-- Called the player starts and stops resting
function Chitchat:PLAYER_UPDATE_RESTING()
	if (IsResting()) then
		if(Chitchat.db.profile.mode == MODE_TOWN) then
			StartRecording()
		end
	else
		if(Chitchat.db.profile.mode == MODE_TOWN) then
			StopRecording()
		end
	end
end

-------------------
-- PLAYER_REGEN_DISABLED
-- Regen becomes disabled and player enters combat
function Chitchat:PLAYER_REGEN_DISABLED()
	local beginRecord = false
	if(Chitchat.db.profile.mode == MODE_COMBAT) then
		if(UnitIsFriend("target","player")) then
			beginRecord = Chitchat_CheckCombatTarget("targettarget")
		else
			beginRecord = Chitchat_CheckCombatTarget("target")
		end
	end
	
	if(beginRecord) then
		StartRecording()
		combatReply = Chitchat.db.profile.combatAutoReply
	end
end

-------------------
-- PLAYER_REGEN_ENABLED
-- Regen becomes enabled and player leaves combat
function Chitchat:PLAYER_REGEN_ENABLED()
	if(Chitchat.db.profile.mode == MODE_COMBAT) then
		combatReply = false
		StopRecording()
		--MissedWhisper()
	end
end

-------------------
-- CHAT_MSG_WHISPER
-- Handles incoming whispers
 function ChatWhisperMessage(self, event, arg1, arg2, ...)
	local recordName = isRecording
	if(recordName == false) then
		for _, canRecordName in ipairs(Chitchat.db.profile.recordlist) do 
			if(strlower(arg2) == strlower(canRecordName)) then
				recordName = true
				break
			end
		end
	end
	
	if(recordName == true) then
		if(Chitchat_RecordWhisper(arg1, arg2)) then
			return true, arg1, arg2, ...
		end
	end
	
	return false, arg1, arg2, ...
end 

-------------------
-- CHAT_MSG_BN_WHISPER
-- Handles incoming Battle.net whisper
 function ChatBNWhisperMessage(self, evenst, arg1, arg2, ...)
	local record = isRecording
	local recordName = false
	
	if(record == false) then
		for _, canRecordName in ipairs(Chitchat.db.profile.recordlist) do 
			if(strlower(arg2) == strlower(canRecordName)) then
				recordName = true
				break
			end
		end
	end
	
	if(recordName == true) or ((record == true) and (Chitchat.db.profile.rid == true)) then
		if(Chitchat_RecordWhisper(arg1, arg2)) then
			return true, arg1, arg2, ...
		end
	end
	
	return false, arg1, arg2, ...
end 

-------------------------------
-- CHAT_MSG_AFK or CHAT_MSG_DND
-- Handles incoming afk or dnd auto-repsonse
 function ChatStatusMessage(self, event, arg1, arg2, ...)
    if(isRecording) then
		if(Chitchat.db.profile.submode == SUBMODE_HIDE or Chitchat.db.profile.submode == SUBMODE_NOTIFY) then
			return true
		else
			return false, arg1, arg2, ...
		end
    end
	return false, arg1, arg2, ...
end 

------------------
-- CHAT_MSG_SYSTEM
-- Start/Stop recording based on afk/dnd status changes
function ChatSystemMessage(self, event, arg1, arg2, ...)
	-- Unit is AFK
	if (string.find(arg1, string.sub(MARKED_AFK_MESSAGE, 1, string.len(MARKED_AFK_MESSAGE) - 4), 1))  then
		if(Chitchat.db.profile.mode == MODE_AFK or Chitchat.db.profile.mode == MODE_AFKDND) then
			StartRecording()
		end
	-- Unit is DND
	elseif (string.find(arg1, string.sub(MARKED_DND, 1, string.len(MARKED_DND) - 3), 1)) then
		if(Chitchat.db.profile.mode == MODE_DND or Chitchat.db.profile.mode == MODE_AFKDND) then
			StartRecording()
		end
	end
	-- Unit is no longer AFK or DND
	if (arg1 == CLEARED_AFK or arg1 == CLEARED_DND) then
		if(Chitchat.db.profile.mode == MODE_AFK or Chitchat.db.profile.mode == MODE_DND or Chitchat.db.profile.mode == MODE_AFKDND) then
			StopRecording()
		end
	end
	return false, arg1, arg2, ...
end


--------------------------
---------| API |----------
--------------------------


function Chitchat_CheckCombatTarget(unit)
	local class = UnitExists(unit) and UnitClassification(unit) or "normal"
	
	if(class == "normal") then
		return false
	end
	
	if(Chitchat.db.profile.combatmode == MODE_COMBAT_ALL) then
		return true
	end
	
	if(Chitchat.db.profile.combatmode == MODE_COMBAT_BOTH and (class == "worldboss" or class == "elite" or class == "rareelite")) then
		return true
	end
	
	if(Chitchat.db.profile.combatmode == MODE_COMBAT_BOSS and class == "worldboss") then
		return true
	end
	
	if(Chitchat.db.profile.combatmode == MODE_COMBAT_ELITE and (class == "elite" or class == "rareelite")) then
		return true
	end
	
	return false
end

-----------------
-- UnreadColorHex
-- Returns the unread color hex formatted
function Chitchat_UnreadColorHex()
	local unreadColor = format("|cff%02x%02x%02x",Chitchat.db.profile.unreadColor.r*255,Chitchat.db.profile.unreadColor.g*255,Chitchat.db.profile.unreadColor.b*255)
	return unreadColor
end

-----------------
-- StartRecording
-- Start recording whispers
function StartRecording()
	-- Check to make sure it is not running
	if(isRecording == false) then
		isRecording = true
		DEFAULT_CHAT_FRAME:AddMessage(HYPERLINK_CHITCHAT .. L["RECORDING_START"], Chitchat.db.profile.unreadColor.r, Chitchat.db.profile.unreadColor.g, Chitchat.db.profile.unreadColor.b);
	end
end

----------------
-- StopRecording
-- Stop Recording whispers
function StopRecording()
	-- Check to make sure it is running
	if(isRecording) then
		isRecording = false
		-- Will only display missed whispers while setting is Automatic Record
		if(Chitchat.db.profile.mode ~= MODE_ON) then
			MissedWhisper()
		end
	end
end

----------------
-- MissedWhisper
-- Display missed whispers to the chat screen if there is any. 
function MissedWhisper()
	if (Chitchat.db.profile.unread == 1 and Chitchat.db.profile.mode ~= MODE_ON) then
		DEFAULT_CHAT_FRAME:AddMessage(HYPERLINK_CHITCHAT .. L["MISSED_MSG_FIRST"] .. Chitchat.db.profile.unread .. L["MISSED_MSG_LAST_SINGULAR"] .. HYPERLINK, Chitchat.db.profile.unreadColor.r, Chitchat.db.profile.unreadColor.g, Chitchat.db.profile.unreadColor.b);
	elseif (Chitchat.db.profile.unread > 1 and Chitchat.db.profile.mode ~= MODE_ON) then
		DEFAULT_CHAT_FRAME:AddMessage(HYPERLINK_CHITCHAT .. L["MISSED_MSG_FIRST"] .. Chitchat.db.profile.unread .. L["MISSED_MSG_LAST_PLURAL"] .. HYPERLINK, Chitchat.db.profile.unreadColor.r, Chitchat.db.profile.unreadColor.g, Chitchat.db.profile.unreadColor.b);
	end
end

----------------
-- RecordWhisper
-- Records the whisper and preform the submode
function Chitchat_RecordWhisper(message, name)
	local cTime = date("%X")
	local cDate = date("%x")
	local hName = format("|Hplayer:%s|h[%s]|h", name, name)
	local timeMsg = format("[%s]%s: %s",cTime, hName, message)
	
	-- Spam Prevention, if a second has not passed between the last whisper it will not record.
	if(lastWhisper ~= timeMsg) then
		AddWhisper(name,cDate,timeMsg)
		if(Chitchat.db.profile.submode == SUBMODE_NOTIFY) then
			PlaySound("TellMessage");
			DEFAULT_CHAT_FRAME:AddMessage(HYPERLINK_CHITCHAT .. " " .. L["NOTIFY_MSG_FIRST"] .. hName .. L["NOTIFY_MSG_LAST"] .. " " .. HYPERLINK , Chitchat.db.profile.unreadColor.r, Chitchat.db.profile.unreadColor.g, Chitchat.db.profile.unreadColor.b);
		end
		
		-- Spam Prevention, saves the last whisper.
		lastWhisper = timeMsg
	end
	if(Chitchat.db.profile.mode == MODE_COMBAT and combatReply) then -- and name ~= GetUnitName("player")) then
		SendChatMessage(L["CHITCHAT"] .. ": " ..Chitchat.db.profile.combatReplyMessage , "WHISPER", nil, name);
	end
	if(Chitchat.db.profile.submode == SUBMODE_HIDE or Chitchat.db.profile.submode == SUBMODE_NOTIFY) then
		return true
	end
	return false
end

-------------
-- AddWhisper
-- Save the whisper
function AddWhisper(author, today, whisper)
	-- NOTE: There must be a cleaner more effient way to do this
	local foundAuthor = false
	local unreadIncrease  = 1
	if(Chitchat.db.profile.mod == MODE_ON) then
		Chitchat.db.profile.unread = 0
		unreadIncrease = 0
	end
	Chitchat.db.profile.unread = Chitchat.db.profile.unread + unreadIncrease
	-- Add whisper
	for _, playerName in ipairs(Chitchat.db.profile.names) do
		if(playerName.playerName == author) then
			playerName.unread = true
			foundAuthor = true
			for _, dated  in ipairs(playerName.dates) do 
				if(dated.theDate == today) then
					dated.unread = true
					if(dated.unreadCount ~= nil) then
						dated.unreadCount = dated.unreadCount + unreadIncrease
					else
						dated.unreadCount = 1
					end
					if(dated.unreadWhispers ~= nil) then
						local newWhispers = format("%s\n%s", dated.unreadWhispers, whisper)
						dated.unreadWhispers = newWhispers
					else
						dated.unreadWhispers = whisper
					end
					Chitchat:SendMessage("CHITCHAT_MSG_WHISPERS_UPDATED")
					return
				end
			end
		end
	end
	-- Add new date column
	if(foundAuthor) then
		for _, playerName in ipairs(Chitchat.db.profile.names) do
			if(playerName.playerName == author) then
				local newDates = { unread = true, theDate = today, whispers = "", unreadWhispers = whisper, unreadCount = unreadIncrease }
				tinsert(playerName.dates, 1, newDates)
				Chitchat:SendMessage("CHITCHAT_MSG_WHISPERS_UPDATED")
				return
			end
		end 
	-- Add new author and date
	else
		local newAuthor = {playerName = author, unread = true, dates = {{ unread = true, theDate = today, whispers = "", unreadWhispers = whisper, unreadCount = unreadIncrease }}}
		tinsert(Chitchat.db.profile.names, 1, newAuthor)
		Chitchat:SendMessage("CHITCHAT_MSG_WHISPERS_UPDATED")
		return
	end
	
end


--------------------------
--------| OPTIONS |-------
--------------------------
---------------
-- ChatCommand
-- Handles chat command input
function Chitchat:ChatCommand(input)
    if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToCategory(Chitchat.optionsFrame)
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(Chitchat, "chitchat", "Chitchat", input)
    end
end

---------------------------------
-- ToggleChitchatFrame
-- Shows or hides the Answering Machine Frame
function Chitchat:ToggleChitchatFrame()
	if ChitchatFrame:IsShown() then
		HideUIPanel(ChitchatFrame)
	else
		ShowUIPanel(ChitchatFrame)
	end
end

------------
-- ClearLogs
-- Clears all entries
function Chitchat:ClearLogs()
	Chitchat.db.profile.unread = 0
	wipe(Chitchat.db.profile.names)
	Chitchat:SendMessage("CHITCHAT_MSG_LOGS_CLEARED")
end

--------------
-- ShowOptions
-- Shows Options Panel
function Chitchat:ShowOptions(info)
	InterfaceOptionsFrame_OpenToCategory(Chitchat.optionsFrame)
end

------------------
-- Get/Set Minimap
-- Shows or hides the Minimap icon
function Chitchat:GetMinimap(info)
    return not self.db.profile.minimap.hide
end
function Chitchat:SetMinimap(info, value)
    self.db.profile.minimap.hide = not value
	if (self.db.profile.minimap.hide) then
		icon:Hide("Chitchat")
	else
		icon:Show("Chitchat")
	end
end

------------------------
-- Is/Toggle AFK Message
-- Sets and Returns the AFK message
function Chitchat:IsAFKEnabled(info)
	if(type(self.db.profile.afk) == "string") then
		self.db.profile.afk = { enable = false, message = self.db.profile.afk }
	end
    return self.db.profile.afk.enable
end
function Chitchat:ToggleAFKMessage(info, newValue)
    self.db.profile.afk.enable = newValue
end

--------------
-- Get/Set AFK
-- Sets and Returns the AFK message
function Chitchat:GetAFK(info)
    return self.db.profile.afk.message
end
function Chitchat:SetAFK(info, newValue)
    self.db.profile.afk.message = newValue
end

------------------------
-- Is/Toggle DND Message
-- Sets and Returns the DND message
function Chitchat:IsDNDEnabled(info)
	if(type(self.db.profile.dnd) == "string") then
		self.db.profile.dnd = { enable = false, message = self.db.profile.dnd }
	end
    return self.db.profile.dnd.enable
end
function Chitchat:ToggleDNDMessage(info, newValue)
    self.db.profile.dnd.enable = newValue
end

--------------
-- Get/Set DND
-- Sets and Returns the DND message
function Chitchat:GetDND(info)
    return self.db.profile.dnd.message
end
function Chitchat:SetDND(info, newValue)
    self.db.profile.dnd.message = newValue
end


------------------------
-- Is/Toggle Real ID Record
-- Sets and Returns Real ID Record
function Chitchat:IsRIDEnabled(info)
    return self.db.profile.rid
end
function Chitchat:ToggleRIDMessage(info, newValue)
    self.db.profile.rid = newValue
end

-------------------------
-- Get/Set/Add Recordlist
-- Sets, Returns and Adds names on the recordlist
function Chitchat:GetRecordlist(info)
    return self.db.profile.recordlist
end
function Chitchat:SetRecordlist(info, newValue)
	-- The value is the index of the name to be removed
	tremove(self.db.profile.recordlist, newValue)
end
function Chitchat:AddRecordlist(info, newValue)
	tinsert(self.db.profile.recordlist, newValue)
end

---------------
-- Get/Set Mode
-- Sets and Returns the mode
function Chitchat:GetMode(info)
	return self.db.profile.mode
end
function Chitchat:SetMode(info, value)
    self.db.profile.mode = value
	
	if(value == MODE_OFF) then
		StopRecording()
	elseif(value == MODE_AFK) then
		if(UnitIsAFK("player")) then
			StartRecording()
		else
			StopRecording()
		end
	elseif(value == MODE_DND) then
		if(UnitIsDND("player")) then
			StartRecording()
		else
			StopRecording()
		end
	elseif(value == MODE_AFKDND) then
		if(UnitIsAFK("player") or UnitIsDND("player")) then
			StartRecording()
		else
			StopRecording()
		end
	elseif(value == MODE_TOWN) then
		if(IsResting()) then
			StartRecording()
		else
			StopRecording()
		end
	elseif(value == MODE_ON) then
		StartRecording()
	end
end

------------------------
-- Is/Toggle Combat Message
-- Sets and Returns the Combat message
function Chitchat:IsCombatAutoreply(info)
    return self.db.profile.combatAutoReply
end
function Chitchat:ToggleCombatAutoreply(info, newValue)
    self.db.profile.combatAutoReply = newValue
end

--------------
-- Get/Set Combat
-- Sets and Returns the Combat message
function Chitchat:GetCombatReplyMessage(info)
    return self.db.profile.combatReplyMessage
end
function Chitchat:SetCombatReplyMessage(info, newValue)
    self.db.profile.combatReplyMessage = newValue
end

------------------------------
-- Get/Set/Visible Combat Mode
-- Sets and Gets type of combat recording, and returns if its visible
function Chitchat:GetModeCombat(info)
    return self.db.profile.combatmode
end
function Chitchat:SetModeCombat(info, value)
	self.db.profile.combatmode = value
end
function Chitchat:VisibleModeCombat(info)
	if(self.db.profile.mode == MODE_COMBAT) then
		return false
	else
		return true
	end
end

------------------
-- Get/Set Submode
-- Sets and Returns the submode
function Chitchat:GetSubmode(info)
	return self.db.profile.submode
end
function Chitchat:SetSubmode(info, value)
    self.db.profile.submode = value
end

------------------
-- Get/Set UnreadColor
-- Sets and Returns the UnreadColor
function Chitchat:GetUnreadColor(info)
	return self.db.profile.unreadColor.r, self.db.profile.unreadColor.g, self.db.profile.unreadColor.b, self.db.profile.unreadColor.a
end
function Chitchat:SetUnreadColor(info, r, g, b, a)
    self.db.profile.unreadColor.r = r
	self.db.profile.unreadColor.b = b
	self.db.profile.unreadColor.g = g
	self.db.profile.unreadColor.a = a
end

--------------------------
-- Get/Set Blacklist Level
-- Sets and Returns the min blacklist level
function Chitchat:GetBlacklistLevel(info)
	return self.db.profile.blacklist.min
end
function Chitchat:SetBlacklistLevel(info, value)
	self.db.profile.blacklist.min = value
end

----------------------------
-- Is/Toggle Blacklist Reply
-- Enables/Disable the blacklist autoreply
function Chitchat:IsBlackReplyEnabled(info)
	return self.db.profile.blacklist.autoreply
end
function Chitchat:ToggleBlackReplyMessage(info, value)
	self.db.profile.blacklist.autoreply = value
end

----------------------------------
-- Get/Set Blacklist Reply Message
-- Sets and Returns blacklist reply message
function Chitchat:GetBlacklistReply(info)
	return self.db.profile.blacklist.message
end
function Chitchat:SetBlacklistReply(info, value)
	self.db.profile.blacklist.message = value
end
